// PrestoConfig.cs
//
// PRESTO Starter Kit for Microsoft .NET 3.0
// 2007 Microsoft France
// Published under the CECILL-B Free Software license agreement.
// (http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.txt)
// 
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// WHETHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE. 
// THE ENTIRE RISK OF USE OR RESULTS IN CONNECTION WITH THE USE OF THIS CODE 
// AND INFORMATION REMAINS WITH THE USER. 
//

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.ComponentModel;
using System.ServiceModel.Security.Tokens;

using Microsoft.Presto.StarterKit.Constants;
using System.ServiceModel.Security;

namespace Microsoft.Presto.StarterKit
{    
    public class PrestoSoapNodeConfig : INotifyPropertyChanged
    {
        // PRESTO Payload information
        private string documentsDirectory = Directory.GetCurrentDirectory() + @"\"; // Payloads.documentsDirectory;
        public string DocumentsDirectory
        {
            get { return documentsDirectory; }
            set { documentsDirectory = value; }
        }

        private string documentToSend = Payloads.documentToSend;
        public string DocumentToSend
        {
            get { return documentToSend; }
            set { documentToSend = value; }
        }
        
        // PRESTO Endpoints addressing information
        private int maxMessageSize = Numbers.maxMessageSize;

        private string serviceEndpointMexUri; // Set in the (Client & Service) App.config file. By default, http://localhost:9999
        public string ServiceEndpointMexUri
        {
            get { return serviceEndpointMexUri; }
            set { serviceEndpointMexUri = value; }
        }

        private string serviceEndpointUri; // Set in the (Client & Service) App.config file. By default, http://localhost:8080
        public string ServiceEndpointUri
        {
            get { return serviceEndpointUri; }
            set { serviceEndpointUri = value; }
        }

        private bool useSoapIntermediary = false;
        public bool UseSoapIntermediary
        {
            get { return useSoapIntermediary; }
            set { useSoapIntermediary = value; }
        }

        // When turned on, automatically add the endpoint Uri suffix to serviceEndpointUri and useSoapIntermediary
        private bool autoCompleteUris = true;
        public bool AutoCompleteUris
        {
            get { return autoCompleteUris; }
            set { autoCompleteUris = value; }
        }

        private string soapIntermediaryEndpointUri; // Set in the (Client & Service) App.config file. By default, http://localhost:9090
        public string SoapIntermediaryEndpointUri
        {
            get { return soapIntermediaryEndpointUri; }
            set { soapIntermediaryEndpointUri = value; }
        }

        // WS-MetadataExchange support (not required nor optional in the current specifications of PRESTO)
        private bool useMetadataExchange = false;
        public bool UseMetadataExchange
        {
            get { return useMetadataExchange; }
            set { useMetadataExchange = value; }
        }
        
        // Configuration file information
        private bool useConfigurationFile = false;
        public bool UseConfigurationFile
        {
            get { return useConfigurationFile; }
            set { useConfigurationFile = value; }
        }

        // Configuration file information
        private bool useManualSettings = false;
        public bool UseManualSettings
        {
            get { return useManualSettings; }
            set { useManualSettings = value; }
        }

        // WS-Addressing version
        private PrestoAddressingVersion addressingVersion = PrestoAddressingVersion.WSAddressing10;
        public PrestoAddressingVersion AddressingVersion
        {
            get { return addressingVersion; }
            set { addressingVersion = value; OnPropertyChanged("AddressingVersion"); }
        }
    
        // WS-ReliableMessaging support
        private bool useReliableMessaging = false;
        public bool UseReliableMessaging
        {
            get { return useReliableMessaging; }
            set { useReliableMessaging = value; }
        }

        // XOP/MTOM support
        private PrestoEncodingMode encodingMode = PrestoEncodingMode.Text;
        public PrestoEncodingMode EncodingMode
        {
            get { return encodingMode; }
            set { encodingMode = value; }
        }


        // WS-Security - XMLENC support (signing and encryption)
        private bool useEncryption = false;
        public bool UseEncryption
        {
            get { return useEncryption; }
            set { useEncryption = value; }
        }


        // WS-Security - XMLDSIG support (signing)
        private bool useDigitalSignature = false;
        public bool UseDigitalSignature
        {
            get { return useDigitalSignature; }
            set { useDigitalSignature = value; }
        }

        // WS-Security - Authentication Mode - NoAuthentication
        private bool noAuthentication = true;
        public bool NoAuthentication
        {
            get { return noAuthentication; }
            set { noAuthentication = value; }
        }

        // WS-Security - Authentication Mode - AnonymousForCertificate
        private bool anonymousForCertificate = false;
        public bool AnonymousForCertificate
        {
            get { return anonymousForCertificate; }
            set { anonymousForCertificate = value; }
        }

        // WS-Security - Authentication Mode - MutualCertificate
        private bool mutualCertificate = false;
        public bool MutualCertificate
        {
            get { return mutualCertificate; }
            set { mutualCertificate = value; }
        }
        // WS-Security - Presto Starter Kit X.509 Test Certificates
	    //    . PRESTO Test Root CA Certificate
        // 	-> SubjectName = "CN = PRESTO Test Root CA"
	    // 	-> Serialnumber = "fd7f3e1d5d15028542ae2e75e1d521af"
	    // 	-> Thumbprint = "5f261135e05008274fd2660e117cc51979fa1bd6"

	    //    . PRESTO Source Application Certificate
        // 	-> SubjectName = "CN = PRESTO Source Application"
	    // 	-> Serialnumber = "151a8c12759f139f4ea63c86536ea77a"
	    // 	-> Thumbprint = "f31416e1d645367ac2db89e31b8ef89a4e77cf21"

	    //    . PRESTO Target Application Certificate
        // 	-> SubjectName = "CN = PRESTO Target Application"
	    // 	-> Serialnumber = "1c237d3d3e04edb340ca8979c8eb5c68"
	    // 	-> Thumbprint = "dc07a1a56e0e6f4e0809999149acc98c8a6e7479"

	    //    . PRESTO Sender Proxy Certificate
        // 	-> SubjectName = "CN = PRESTO Sender Proxy"
	    // 	-> Serialnumber = "c17826da5eec96944d589ff2dacefeb4"
	    // 	-> Thumbprint = "5e151badbd8925769f88487fc5b49ff02b0b1d67"

	    //    . PRESTO Receiver Proxy Certificate
        // 	-> SubjectName = "CN = PRESTO Receiver Proxy"
	    // 	-> Serialnumber = "86fd2a555717a6b54d90281a9b7ab310"
	    // 	-> Thumbprint = "cc73fe09c209e7216fcd22e90637db0705f1b8e1"

        private string clientCertificateThumbprint; // Set in the (Client) App.config file
        public string ClientCertificateThumbprint
        {
            get { return clientCertificateThumbprint; }
            set { clientCertificateThumbprint = value; }
        }

        private string serviceIdentity; // Set in the (Client) App.config file 
        public string ServiceIdentity
        {
            get { return serviceIdentity; }
            set { serviceIdentity = value; }
        }

        private string serviceCertificateThumbprint; // Set in the (Client & Service) App.config file
        public string ServiceCertificateThumbprint
        {
            get { return serviceCertificateThumbprint; }
            set { serviceCertificateThumbprint = value; }
        }

        public Binding GenerateBinding()
        {
            List<BindingElement> elements = new List<BindingElement>();

            // WS-ReliableMessaging support
            if (this.useReliableMessaging)
            {
                ReliableSessionBindingElement reliableSessionBindingElement = null;
                reliableSessionBindingElement = new ReliableSessionBindingElement();
                reliableSessionBindingElement.AcknowledgementInterval = new TimeSpan(0, 0, 0, 2);
                reliableSessionBindingElement.MaxTransferWindowSize = 32;
                //reliableSessionBindingElement.InactivityTimeout = new TimeSpan(0, 0, 10);
                reliableSessionBindingElement.InactivityTimeout = TimeSpan.FromMinutes(2);
                reliableSessionBindingElement.MaxPendingChannels = 32;
                reliableSessionBindingElement.MaxRetryCount = 8;
                reliableSessionBindingElement.FlowControlEnabled = true;
                reliableSessionBindingElement.Ordered = true;
                elements.Add(reliableSessionBindingElement);
            }

            // WS-Security support : 2 authentication Modes implemented in this prototype
            if (this.anonymousForCertificate)
            {
                // No need for mutual authentication, so no need for endorsing client certificate
                SymmetricSecurityBindingElement messageSecurity = SecurityBindingElement.CreateAnonymousForCertificateBindingElement();
                
                // This method creates a CustomBinding using a SymmetricSecurityBindingElement.
                // It is largely equivalent to doing the following:
                // WSHttpBinding messageSecurity = new WSHttpBinding ( SecurityMode.Message );
                // messageSecurity.Security.Message.ClientCredentialType = MessageCredentialType.None;
                // messageSecurity.Security.Message.NegotiateServiceCredential = false;
                // messageSecurity.Security.Message.EstablishSecureSession = false;
             
                // It differs in that it uses MessageProtectionOrder.SignBeforeEncrypt rather
                // than MessageProtectionOrder.SignBeforeEncryptAndEncryptSignature.
                // Set MessageProtectionOrder to SignBeforeEncrypt.
                messageSecurity.MessageProtectionOrder = MessageProtectionOrder.SignBeforeEncrypt;

                // This would be an alternative for the MutualCertificate authentication mode
                // It is better to use CreateMutualCertificateBindingElement
                // Create supporting token parameters for the client X509 certificate.
                //X509SecurityTokenParameters clientX509SupportingTokenParameters = new X509SecurityTokenParameters();
                //// Specify that the supporting token is passed in message send by the client to the service
                //clientX509SupportingTokenParameters.InclusionMode = SecurityTokenInclusionMode.AlwaysToRecipient;
                //// Turn off derived keys
                //clientX509SupportingTokenParameters.RequireDerivedKeys = false;
                //// Augment the binding element to require the client's X509 certificate as an endorsing token in the message
                //messageSecurity.EndpointSupportingTokenParameters.Endorsing.Add(clientX509SupportingTokenParameters);
                
                // Set the algorithm suite to one that uses 256-bit keys.
                messageSecurity.DefaultAlgorithmSuite = SecurityAlgorithmSuite.Basic256;
                
                elements.Add(messageSecurity);
            }
            else if (this.MutualCertificate)
            {
                SecurityBindingElement messageSecurity = SecurityBindingElement.CreateMutualCertificateBindingElement();
            
                // Set the algorithm suite to one that uses 256-bit keys.
                messageSecurity.DefaultAlgorithmSuite = SecurityAlgorithmSuite.Basic256;

                elements.Add(messageSecurity);
            }

            // Text encoding or XOP/MTOM
            //To add
            //POSX
            //textEncodingBindingElement.MessageVersion = MessageVersion.None;
                
            MessageEncodingBindingElement encodingBindingElement = null;
            if (this.EncodingMode == PrestoEncodingMode.Text)
            {
                TextMessageEncodingBindingElement textEncodingBindingElement = new TextMessageEncodingBindingElement();
                textEncodingBindingElement.WriteEncoding = System.Text.Encoding.UTF8;
                textEncodingBindingElement.ReaderQuotas.MaxArrayLength = maxMessageSize;
                encodingBindingElement = textEncodingBindingElement;
            }    
            else
            {
                MtomMessageEncodingBindingElement mtomEncodingBindingElement = new MtomMessageEncodingBindingElement();
                mtomEncodingBindingElement.WriteEncoding = System.Text.Encoding.UTF8;
                mtomEncodingBindingElement.ReaderQuotas.MaxArrayLength = maxMessageSize; 
                encodingBindingElement = mtomEncodingBindingElement;
            }

            // WS-Addressing version
            if (this.AddressingVersion == PrestoAddressingVersion.WSAddressing10)
            {
                encodingBindingElement.MessageVersion = MessageVersion.Soap12WSAddressing10;
            }
            else
            {
                encodingBindingElement.MessageVersion = MessageVersion.Soap12WSAddressingAugust2004;
            }
            elements.Add(encodingBindingElement);

            // HTTP transport
            HttpTransportBindingElement httpTransportBindingElement = new HttpTransportBindingElement();
            httpTransportBindingElement.ManualAddressing = false;
            httpTransportBindingElement.MaxReceivedMessageSize = maxMessageSize; 
            httpTransportBindingElement.AllowCookies = true;
            httpTransportBindingElement.AuthenticationScheme = System.Net.AuthenticationSchemes.Anonymous;
            httpTransportBindingElement.HostNameComparisonMode = HostNameComparisonMode.StrongWildcard;
            httpTransportBindingElement.ProxyAuthenticationScheme = System.Net.AuthenticationSchemes.Anonymous;
            httpTransportBindingElement.Realm = "";
            httpTransportBindingElement.TransferMode = TransferMode.Buffered;
            httpTransportBindingElement.UnsafeConnectionNtlmAuthentication = false;
            httpTransportBindingElement.UseDefaultWebProxy = true;
            elements.Add(httpTransportBindingElement);

            CustomBinding currentBinding = new CustomBinding(elements.ToArray());
            return currentBinding;
        }

        // Return a defined binding name within the configuration file based on the current UI settings 
        public string GetBindingNameFromConfigurationFile()
        {
            if ((this.UseReliableMessaging) && (this.EncodingMode == PrestoEncodingMode.Mtom))
            {
                return "prestoMtomReliableBinding";
            }
            if ((this.UseReliableMessaging) && (this.EncodingMode == PrestoEncodingMode.Text))
            {
                return "prestoReliableBinding";
            }
            if ((!this.UseReliableMessaging) && (this.EncodingMode == PrestoEncodingMode.Mtom))
            {
                return "prestoMtomBinding";
            }
            if ((!this.UseReliableMessaging) && (this.EncodingMode == PrestoEncodingMode.Text))
            {
                return "prestoBinding";
            }
            throw new Exception("Unable to locate the related configuration in the App.config file");
        }

        // Return a defined binding name within the configuration file based on the current UI settings 
        public string GetEndpointNameFromConfigurationFile(bool isOneWayMEP)
        {
            if ((this.UseReliableMessaging) && (this.EncodingMode == PrestoEncodingMode.Mtom))
            {
                return (isOneWayMEP) ? "prestoMtomReliable" : "presto2MtomReliable";
            }
            if ((this.UseReliableMessaging) && (this.EncodingMode == PrestoEncodingMode.Text))
            {
                return (isOneWayMEP) ? "prestoReliable" : "presto2Reliable";
            }
            if ((!this.UseReliableMessaging) && (this.EncodingMode == PrestoEncodingMode.Mtom))
            {
                return (isOneWayMEP) ? "prestoMtom" : "presto2Mtom";
            }
            if ((!this.UseReliableMessaging) && (this.EncodingMode == PrestoEncodingMode.Text))
            {
                return (isOneWayMEP) ? "presto" : "presto2";
            }
            throw new Exception("Unable to locate the related configuration in the App.config file");
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;
        private void OnPropertyChanged(string propertyname)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyname));
        }
        #endregion
    }
}
